#!/usr/bin/env python

"""
globewidget.py

A custom widget for displaying geographical locations.

Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
Contact: Qt Software Information (qt-info@nokia.com)

This file is part of the documentation of Qt. It was originally
published as part of Qt Quarterly.

Commercial Usage
Licensees holding valid Qt Commercial licenses may use this file in
accordance with the Qt Commercial License Agreement provided with the
Software or, alternatively, in accordance with the terms contained in
a written agreement between you and Nokia.


GNU General Public License Usage
Alternatively, this file may be used under the terms of the GNU
General Public License versions 2.0 or 3.0 as published by the Free
Software Foundation and appearing in the file LICENSE.GPL included in
the packaging of this file.  Please review the following information
to ensure GNU General Public Licensing requirements will be met:
http://www.fsf.org/licensing/licenses/info/GPLv2.html and
http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
exception, Nokia gives you certain additional rights. These rights
are described in the Nokia Qt GPL Exception version 1.3, included in
the file GPL_EXCEPTION.txt in this package.

Qt for Windows(R) Licensees
As a special exception, Nokia, as the sole copyright holder for Qt
Designer, grants users of the Qt/Eclipse Integration plug-in the
right for the Qt/Eclipse Integration to link to functionality
provided by Qt Designer and its related libraries.

If you are unsure which license is appropriate for your use, please
contact the sales department at qt-sales@nokia.com.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from math import cos, sin, pi


class GlobeWidget(QWidget):

    def __init__(self, parent = None):
    
        QWidget.__init__(self, parent)
        
        self._angle = 0
        self._divisions = 24
        self._latitude = 0
        self._longitude = 0
        self._position_shown = False
    
    def sizeHint(self):
    
        return QSize(200, 200)
    
    def paintEvent(self, event):
    
        cx = self.width() * 0.5
        cy = self.height() * 0.5
        sx = self.width() * 0.4
        sy = self.height() * 0.4
        size = min(sx, sy)
        
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.fillRect(event.rect(), Qt.white)
        
        step = int(360.0 / self._divisions)
        
        for latitude in range(-90 + step, 90, step):
            a = cos(latitude/180.0 * pi)
            y = sin(latitude/180.0 * pi)
            for angle in range(90 - (self._angle % step), -90, -step):
                x = sin(angle/180.0 * pi) * a
                z = cos(angle/180.0 * pi) * a
                scrx = cx + (size * x) * (1 + 0.1 * z)
                scry = cy - (size * y) * (1 + 0.1 * z)
                painter.drawEllipse(scrx - 1, scry - 1, 2, 2)
        
        rect = QRectF(cx - size - 2, cy - size - 2, size * 2 + 4, size * 2 + 4)
        
        pen = QPen(Qt.DashLine)
        pen.setWidth(2)
        painter.setPen(pen)
        painter.drawEllipse(rect)
        
        for special_angle in (-180, 0, 180):
        
            if special_angle - 90 < self._angle < special_angle + 90:
            
                b = sin((special_angle - self._angle)/180.0 * pi)
                c = cos((special_angle - self._angle)/180.0 * pi)
                
                points = []
                
                for latitude in range(-90, 91):
                
                    a = cos(latitude/180.0 * pi)
                    x = a * b
                    z = a * c
                    y = sin(latitude/180.0 * pi)
                    scrx = cx + (size * x) * (1 + 0.1 * z)
                    scry = cy - (size * y) * (1 + 0.1 * z)
                    points.append(QPointF(scrx, scry))
                
                painter.drawPolyline(*points)
        
        if self._position_shown:
        
            angle = (self._longitude - self._angle) % 360
            
            if angle < 90 or angle > 270:
            
                a = cos(self._latitude/180.0 * pi)
                y = sin(self._latitude/180.0 * pi)
                x = sin((self._longitude - self._angle)/180.0 * pi) * a
                z = cos((self._longitude - self._angle)/180.0 * pi) * a
                scrx = cx + (size * x) * (1 + 0.1 * z)
                scry = cy - (size * y) * (1 + 0.1 * z)
                painter.setPen(Qt.red)
                painter.setBrush(Qt.red)
                painter.drawEllipse(scrx - 2, scry - 2, 4, 4)
        
        painter.end()
    
    def angle(self):
        return self._angle
    
    @pyqtSignature("setAngle(int)")
    def setAngle(self, angle):
        self._angle = max(-180, min(angle, 180))
        self.update()
    
    angle = pyqtProperty("int", angle, setAngle)
    
    def divisions(self):
        return self._divisions
    
    @pyqtSignature("setDivisions(int)")
    def setDivisions(self, divisions):
        if 4 <= divisions <= 360:
            self._divisions = divisions
            self.update()
    
    divisions = pyqtProperty("int", divisions, setDivisions)
    
    def latitude(self):
        return self._latitude
    
    @pyqtSignature("setLatitude(double)")
    def setLatitude(self, latitude):
        self._latitude = max(-90.0, min(latitude, 90.0))
        if self.positionShown:
            self.update()
    
    latitude = pyqtProperty("double", latitude, setLatitude)
    
    def longitude(self):
        return self._longitude
    
    @pyqtSignature("setLongitude(double)")
    def setLongitude(self, longitude):
        self._longitude = max(-180.0, min(longitude, 180.0))
        if self.positionShown:
            self.update()
    
    longitude = pyqtProperty("double", longitude, setLongitude)
    
    def positionShown(self):
        return self._position_shown
    
    @pyqtSignature("setPositionShown(bool)")
    def setPositionShown(self, enabled):
        self._position_shown = enabled
        self.update()
    
    positionShown = pyqtProperty("bool", positionShown, setPositionShown)
